package cn.bunny.module.minio.utils;

import cn.bunny.common.constant.MinioMessageConstant;
import cn.bunny.common.service.exception.BunnyException;
import io.minio.*;
import io.minio.messages.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.InputStream;
import java.util.List;
import java.util.Map;

/**
 * Minio操作工具类
 * 简化操作步骤
 */
@Component
@Slf4j
public class MinioUtil {
    @Autowired
    private MinioClient minioClient;

    /**
     * 判断桶是否存在
     *
     * @param bucketName 桶名称
     * @return 布尔值，是否存在
     */
    public boolean bucketExists(String bucketName) {
        boolean found = false;
        try {
            found = minioClient.bucketExists(BucketExistsArgs.builder().bucket(bucketName).build());
            return found;
        } catch (Exception exception) {
            log.error("判断桶是否存在 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
        }
        throw new BunnyException(MinioMessageConstant.BUCKET_EXISTS_EXCEPTION);
    }

    /**
     * 删除桶的加密配置
     *
     * @param bucketName 桶名称
     * @return 是否删除成功，返回布尔值
     */
    public boolean deleteBucketEncryption(String bucketName) {
        try {
            minioClient.deleteBucketEncryption(DeleteBucketEncryptionArgs.builder().build());
            log.info("删除桶的加密配置 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("删除桶的加密配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
        }
        throw new BunnyException(MinioMessageConstant.DELETE_BUCKET_EXCEPTION);
    }

    /**
     * 删除桶的生命周期配置
     *
     * @param bucketName 桶名称
     * @return 返回布尔值，是否删除成功
     */
    public boolean deleteBucketLifecycle(String bucketName) {
        try {
            minioClient.deleteBucketLifecycle(DeleteBucketLifecycleArgs.builder().build());
            log.info("删除桶的生命周期配置 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("删除桶的生命周期配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.DELETE_BUCKET_EXCEPTION);
        }
    }

    /**
     * 删除桶的通知配置
     *
     * @param bucketName 桶的名称
     * @return 返回布尔值是否删除成功
     */
    public boolean deleteBucketNotification(String bucketName) {
        try {
            minioClient.deleteBucketNotification(DeleteBucketNotificationArgs.builder().bucket(bucketName).build());
            log.info("删除桶的通知配置 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("删除桶的加密配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.DELETE_BUCKET_EXCEPTION);
        }
    }

    /**
     * 删除桶策略配置
     *
     * @param bucketName 桶的名称
     * @return 返回布尔值是否删除成功
     */
    public boolean deleteBucketPolicy(String bucketName) {
        try {
            minioClient.deleteBucketPolicy(DeleteBucketPolicyArgs.builder().bucket(bucketName).build());
            log.info("删除桶中的对象锁配置 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("删除桶的加密配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.DELETE_BUCKET_EXCEPTION);
        }
    }

    /**
     * 删除桶的桶复制配置
     *
     * @param bucketName 桶的名称
     * @return 返回布尔值是否删除成功
     */
    public boolean deleteBucketReplication(String bucketName) {
        try {
            minioClient.deleteBucketReplication(DeleteBucketReplicationArgs.builder().bucket(bucketName).build());
            log.info("删除桶的桶复制配置 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("删除桶的桶复制配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.DELETE_BUCKET_EXCEPTION);
        }
    }

    /**
     * 删除桶的标签
     *
     * @param bucketName 桶的名称
     * @return 返回布尔值是否删除成功
     */
    public boolean deleteBucketTags(String bucketName) {
        try {
            minioClient.deleteBucketTags(DeleteBucketTagsArgs.builder().bucket(bucketName).build());
            log.info("删除桶的标签 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("删除桶的桶复制配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.DELETE_BUCKET_EXCEPTION);
        }
    }

    /**
     * 删除桶中的对象锁配置
     *
     * @param bucketName 桶的名称
     * @return 返回布尔值是否删除成功
     */
    public boolean deleteObjectLockConfiguration(String bucketName) {
        try {
            minioClient.deleteObjectLockConfiguration(DeleteObjectLockConfigurationArgs.builder().bucket(bucketName).build());
            log.info("删除桶中的对象锁配置 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("删除桶中的对象锁配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.DELETE_BUCKET_EXCEPTION);
        }
    }

    /**
     * 获取存储桶的加密配置
     *
     * @param bucketName 桶的名称
     * @return SseConfiguration 获取成功不为null
     */
    public SseConfiguration getBucketEncryption(String bucketName) {
        SseConfiguration conf = null;

        try {
            conf = minioClient.getBucketEncryption(GetBucketEncryptionArgs.builder().bucket(bucketName).build());
            log.info("获取存储桶的加密配置 ------ 成功");
        } catch (Exception exception) {
            log.error("获取存储桶的加密配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.GET_BUCKET_EXCEPTION);
        }

        return conf;
    }

    /**
     * 获取存储桶的生命周期配置
     *
     * @param bucketName 桶的名称
     * @return LifecycleConfiguration 获取成功不为null
     */
    public LifecycleConfiguration getBucketLifecycle(String bucketName) {
        LifecycleConfiguration lifecycle = null;
        try {
            lifecycle = minioClient.getBucketLifecycle(GetBucketLifecycleArgs.builder().bucket(bucketName).build());
            log.info("获取存储桶的生命周期配置 ------ 成功");
        } catch (Exception exception) {
            log.error("获取存储桶的生命周期配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.GET_BUCKET_EXCEPTION);
        }

        return lifecycle;
    }

    /**
     * 获取存储桶的通知配置
     *
     * @param bucketName 桶的名称
     * @return NotificationConfiguration 获取成功不为null
     */
    public NotificationConfiguration getBucketNotification(String bucketName) {
        NotificationConfiguration configuration = null;

        try {
            configuration = minioClient.getBucketNotification(GetBucketNotificationArgs.builder().bucket(bucketName).build());
            log.info("获取存储桶的通知配置 ------ 成功");
        } catch (Exception exception) {
            log.error("获取存储桶的通知配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.GET_BUCKET_EXCEPTION);
        }

        return configuration;
    }

    /**
     * 获取桶的桶策略配置
     *
     * @param bucketName 桶的名称
     * @return String 获取成功不为null
     */
    public String getBucketPolicy(String bucketName) {
        String config = null;

        try {
            config = minioClient.getBucketPolicy(GetBucketPolicyArgs.builder().bucket(bucketName).build());
            log.info("获取桶的桶策略配置 ------ 成功");
        } catch (Exception exception) {
            log.error("获取桶的桶策略配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.GET_BUCKET_EXCEPTION);
        }

        return config;
    }

    /**
     * 获取桶复制参数
     *
     * @param bucketName 桶的名称
     * @return ReplicationConfiguration 获取成功不为null
     */
    public ReplicationConfiguration getBucketReplication(String bucketName) {
        ReplicationConfiguration configuration = null;

        try {
            configuration = minioClient.getBucketReplication(GetBucketReplicationArgs.builder().bucket(bucketName).build());
            log.info("获取桶复制参数 ------ 成功");
        } catch (Exception exception) {
            log.error("获取桶复制参数 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.GET_BUCKET_EXCEPTION);
        }

        return configuration;
    }

    /**
     * 获取存储桶的标签
     *
     * @param bucketName 桶的名称
     * @return Tags 获取成功不为null
     */
    public Tags getBucketTags(String bucketName) {
        Tags tags = null;

        try {
            tags = minioClient.getBucketTags(GetBucketTagsArgs.builder().bucket(bucketName).build());
            log.info("获取存储桶的标签 ------ 成功");
        } catch (Exception exception) {
            log.error("获取存储桶的标签 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.GET_BUCKET_EXCEPTION);
        }
        return tags;
    }

    /**
     * 获取存储桶的版本控制配置
     *
     * @param bucketName 桶的名称
     * @return Tags 获取成功不为null
     */
    public VersioningConfiguration getBucketVersioning(String bucketName) {
        VersioningConfiguration configuration = null;

        try {
            configuration = minioClient.getBucketVersioning(GetBucketVersioningArgs.builder().bucket(bucketName).build());
            log.info("获取存储桶的版本控制配置 ------ 成功");
        } catch (Exception exception) {
            log.error("获取存储桶的版本控制配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.GET_BUCKET_EXCEPTION);
        }

        return configuration;
    }

    /**
     * 获取存储桶中的对象锁配置
     *
     * @param bucketName 桶的名称
     * @return Tags 获取成功不为null
     */
    public ObjectLockConfiguration getObjectLockConfiguration(String bucketName) {
        ObjectLockConfiguration configuration = null;
        try {
            configuration = minioClient.getObjectLockConfiguration(GetObjectLockConfigurationArgs.builder().bucket(bucketName).build());
            log.info("获取存储桶中的对象锁配置 ------ 成功");
        } catch (Exception exception) {
            log.error("获取存储桶中的对象锁配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.GET_BUCKET_EXCEPTION);
        }

        return configuration;
    }

    /**
     * 列出所有桶的桶信息
     *
     * @return Tags 获取成功不为null
     */
    public List<Bucket> listBuckets() {
        List<Bucket> buckets = null;

        try {
            buckets = minioClient.listBuckets();
            log.info("列出所有桶的桶信息 ------ 成功");
        } catch (Exception exception) {
            log.error("列出所有桶的桶信息 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.QUERY_BUCKET_EXCEPTION);
        }

        return buckets;
    }

    /**
     * 创建桶
     *
     * @param bucketName 桶的名称
     * @return boolean 返回是否创建成功
     */
    public boolean makeBucket(String bucketName) {
        try {
            minioClient.makeBucket(MakeBucketArgs.builder().bucket(bucketName).build());
            log.info("创建桶 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("创建桶 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.CREATE_BUCKET_EXCEPTION);
        }
    }

    /**
     * 创建桶
     *
     * @param bucketName 桶的名称
     * @return boolean 返回是否创建成功
     */
    public boolean makeBucket(String bucketName, String region) {
        try {
            minioClient.makeBucket(MakeBucketArgs.builder().bucket(bucketName).region(region).build());
            log.info("创建桶 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("创建桶 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.CREATE_BUCKET_EXCEPTION);
        }
    }

    /**
     * 创建桶
     *
     * @param bucketName 桶的名称
     * @return boolean 返回是否创建成功
     */
    public boolean makeBucket(String bucketName, String region, boolean objectLock) {
        try {
            minioClient.makeBucket(MakeBucketArgs.builder().bucket(bucketName).region(region).objectLock(objectLock).build());
            log.info("创建桶 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("创建桶 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.CREATE_BUCKET_EXCEPTION);
        }
    }

    /**
     * 删除桶
     *
     * @param bucketName 桶的名称
     * @return boolean 是否删除成功
     */
    public boolean removeBucket(String bucketName) {
        try {
            minioClient.removeBucket(RemoveBucketArgs.builder().bucket(bucketName).build());
            log.info("删除桶 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("删除桶 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.DELETE_BUCKET_EXCEPTION);
        }
    }

    /**
     * 设置桶的加密配置
     *
     * @param bucketName    同名称
     * @param configuration 配置信息
     * @return 布尔值是否完成
     */
    public boolean setBucketEncryption(String bucketName, SseConfiguration configuration) {
        try {
            minioClient.setBucketEncryption(SetBucketEncryptionArgs.builder()
                    .bucket(bucketName)
                    .config(configuration)
                    .build());
            log.info("设置桶的加密配置 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("设置桶的加密配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.UPDATE_BUCKET_EXCEPTION);
        }
    }

    /**
     * 设置桶策略
     *
     * @param bucketName 同名称
     * @param policyJson 配置信息
     * @return 布尔值是否完成
     */
    public boolean setBucketPolicy(String bucketName, String policyJson) {
        try {
            minioClient.setBucketPolicy(SetBucketPolicyArgs.builder().bucket(bucketName).config(policyJson).build());
            log.info("查看桶策略 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("查看桶策略 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.UPDATE_BUCKET_EXCEPTION);
        }
    }

    /**
     * 设置桶复制参数
     *
     * @param bucketName 同名称
     * @param config     配置信息
     * @return 布尔值是否完成
     */
    public boolean setBucketReplication(String bucketName, ReplicationConfiguration config) {
        try {
            minioClient.setBucketReplication(SetBucketReplicationArgs.builder().bucket(bucketName).config(config).build());
            log.info("设置桶复制参数 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("设置桶复制参数 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.UPDATE_BUCKET_EXCEPTION);
        }
    }

    /**
     * 设置/修改桶标签
     *
     * @param bucketName 桶名称
     * @param map        Map<String, String> map集合
     * @return 布尔值是否完成
     */
    public boolean setBucketTags(String bucketName, Map<String, String> map) {
        try {
            minioClient.setBucketTags(SetBucketTagsArgs.builder().bucket(bucketName).tags(map).build());
            log.info("设置/修改桶标签 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("设置/修改桶标签 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.UPDATE_BUCKET_EXCEPTION);
        }
    }

    /**
     * 设置桶的版本配置
     *
     * @param bucketName    同名称
     * @param configuration VersioningConfiguration 配置信息
     * @return 布尔值是否完成
     */
    public boolean setBucketVersioning(String bucketName, VersioningConfiguration configuration) {
        try {
            minioClient.setBucketVersioning(
                    SetBucketVersioningArgs.builder().bucket(bucketName).config(configuration).build());
            log.info("设置桶的版本配置 ------ 成功");
            return true;
        } catch (Exception exception) {
            log.error("设置桶的版本配置 ------ 失败消息:{}", exception.getLocalizedMessage());
            exception.getStackTrace();
            throw new BunnyException(MinioMessageConstant.UPDATE_BUCKET_EXCEPTION);
        }
    }

    // TODO setObjectLockConfiguration
    // TODO https://min.io/docs/minio/linux/developers/java/API.html#getBucketTags

    /**
     * 复制另一个桶中文件
     *
     * @param bucketName       桶中名字
     * @param objectName       复制名称
     * @param sourceBucketName 复制到桶中名称
     * @param sourceObjectName 复制完成后名称
     * @return 布尔值是否完成
     */
    public boolean copyObject(String bucketName, String objectName, String sourceBucketName, String sourceObjectName) {
        boolean isComplete = false;

        try {
            minioClient.copyObject(
                    CopyObjectArgs.builder()
                            .bucket(bucketName)
                            .object(objectName)
                            .source(
                                    CopySource.builder()
                                            .bucket(sourceBucketName)
                                            .object(sourceObjectName)
                                            .build())
                            .build());
            isComplete = true;
        } catch (Exception exception) {
            exception.printStackTrace();
        }

        return isComplete;
    }

    /**
     * 删除对象的标记。
     *
     * @param bucketName 桶中名字
     * @param objectName 复制名称
     * @return 布尔值是否完成
     */
    public boolean deleteObjectTags(String bucketName, String objectName) {
        boolean isComplete = false;

        try {
            minioClient.deleteObjectTags(DeleteObjectTagsArgs.builder().bucket(bucketName).object(objectName).build());
            isComplete = true;
        } catch (Exception exception) {
            exception.printStackTrace();
        }

        return isComplete;
    }

    /**
     * 下载文件
     *
     * @param bucketName 桶名称
     * @param objectName 对象名称
     * @param filename   文件名
     */
    public void downloadObject(String bucketName, String objectName, String filename) {
        try {
            minioClient.downloadObject(
                    DownloadObjectArgs.builder()
                            .bucket(bucketName)
                            .object(objectName)
                            .filename(filename)
                            .build());
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 获取文件
     *
     * @param bucketName 桶名称
     * @param objectName 对象名称
     */
    public InputStream getObject(String bucketName, String objectName) {
        try {
            return minioClient.getObject(GetObjectArgs.builder()
                    .bucket(bucketName)
                    .object(objectName)
                    .build());
        } catch (Exception exception) {
            exception.printStackTrace();
            return null; // 或者抛出自定义异常
        }
    }

    /**
     * 上传文件
     *
     * @param bucketName  桶名称
     * @param filename    文件名
     * @param inputStream 输入流
     * @param size        大小
     */
    public void putObject(String bucketName, String filename, InputStream inputStream, Long size) {
        try {
            minioClient.putObject(
                    PutObjectArgs.builder().bucket(bucketName)
                            .object(filename)
                            .stream(inputStream, size, -1)
                            .build());
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }


    /**
     * 上传文件
     *
     * @param bucketName  桶名称
     * @param filename    文件名
     * @param contentType 文件类型
     * @param inputStream 输入流
     * @param size        大小
     */
    public void putObject(String bucketName, String filename, String contentType, InputStream inputStream, Long size) {
        try {
            minioClient.putObject(
                    PutObjectArgs.builder().bucket(bucketName)
                            .object(filename)
                            .stream(inputStream, size, -1)
                            .contentType(contentType)
                            .build());
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 删除文件
     *
     * @param bucketName 桶名称
     * @param objectName 对象名称
     */
    public void removeObject(String bucketName, String objectName) {
        try {
            minioClient.removeObject(
                    RemoveObjectArgs.builder()
                            .bucket(bucketName)
                            .object(objectName)
                            .build());
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }
}